function varargout = g_kepler(y,H0,L0,varargin)
% G_KEPLER Constraint for Kepler's 2 body problem.
%
%   G = G_KEPLER(Y,H0,L0) computes the value of the manifold constraint
%   for Kepler's 2 body problem. H0 denotes the total energy and L0 the
%   angular momentum.
%
%   [G,GP] = G_KEPLER(Y,H0,L0) also computes the value of the jacobian
%   for Kepler's 2 body problem.
%
%   The manifold constraints ensure that the total energy and optionally
%   the angular momentum are preserved.
%
%   Example:
%
%   [g,gp] = g_kepler(y,H0,L0);

%   Copyright 2005-2005 Patrick Meury
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  q1 = y(1,:);
  q2 = y(2,:);
  p1 = y(3,:);
  p2 = y(4,:);

  nrm_q = sqrt(q1.^2+q2.^2);
  nrm_p = sqrt(p1.^2+p2.^2);
  
  if(nargin > 2)
  
      % Compute values of constraints
  
      varargout{1} = [1/2*nrm_p.^2- 1./nrm_q - H0; ...
                      q1.*p2 - q2.*p1 - L0];
  
      % Compute jacobian if necessary
              
      if(nargout > 1)
        varargout{2} = [q1/nrm_q^3  q2/nrm_q^3  p1 p2; ...
                        p2         -p1         -q2 q1]; 
      end
  
    else
  
     % Compute values of constraints
  
    varargout{1} = 1/2*nrm_p.^2- 1./nrm_q - H0;
  
    % Compute jacobian if necessary
              
    if(nargout > 1)
      varargout{2} = [q1/nrm_q^3  q2/nrm_q^3  p1 p2]; 
    end   
        
  end
  
return