function varargout = g_mp(y,R,varargin)
% G_MP Constraint for the mathematical pendulum.
%
%   G = G_MP(Y,R,0) computes the value of the manifold constraint for the
%   mathematical pendulum. R denotes the pendulum length.
%
%   [G,GP] = G_MP(Y,R,0) also computes the jacobian of the manifold
%   constraints for the mathematical pendulum.
%
%   The manifold constraints ensure that the pendulum length and optionally
%   the orthogonality between position and velocity vectors are preserved.
%
%   Example:
%
%   [g,gp] = g_mp(y,R,0);

%   Copyright 2005-2005 Patrick Meury
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  q1 = y(1,:);
  q2 = y(2,:);
  p1 = y(3,:);
  p2 = y(4,:);
  
  if(nargin > 2)
  
    % Compute values of constraints

    varargout{1} = [q1.^2+q2.^2-R^2; ...
                    q1.*p1+q2.*p2];
  
    % Compute jacobian if necessary
  
    if(nargout > 1)
      varargout{2} = [2*q1 2*q2 0  0; ...
                      p1   p2   q1 q2]; 
    end
  
  else 
  
    % Compute values of constraints

    varargout{1} = q1.^2+q2.^2-R^2;
  
    % Compute jacobian if necessary
  
    if(nargout > 1)
      varargout{2} = [2*q1 2*q2 0  0]; 
    end  
      
  end
  
return